==> view_mode_switch-8.x-1.1.tar.gz: view_mode_switch/src/ViewModeSwitch.php <==
<?php

namespace Drupal\view_mode_switch;

use Drupal\Core\Cache\Cache;
use Drupal\Core\Cache\CacheBackendInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Entity\FieldableEntityInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\view_mode_switch\Entity\EntityFieldManagerInterface;

/**
 * Provides the view mode switch service.
 */
class ViewModeSwitch implements ViewModeSwitchInterface {

  /**
   * The instantiated cache backend.
   *
   * @var \Drupal\Core\Cache\CacheBackendInterface
   */
  protected $cache;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The logger channel factory.
   *
   * @var \Drupal\Core\Logger\LoggerChannelFactoryInterface
   */
  protected $logger;

  /**
   * The view mode switch entity field manager.
   *
   * @var \Drupal\view_mode_switch\Entity\EntityFieldManagerInterface
   */
  protected $viewModeSwitchEntityFieldManager;

  /**
   * Constructs a new ViewModeSwitch.
   *
   * @param \Drupal\view_mode_switch\Entity\EntityFieldManagerInterface $view_mode_switch_field_manager
   *   The view mode switch entity field manager.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\Core\Cache\CacheBackendInterface $cache
   *   A cache backend used to store view mode switches.
   * @param \Drupal\Core\Logger\LoggerChannelFactoryInterface $logger
   *   The logger channel factory.
   */
  public function __construct(EntityFieldManagerInterface $view_mode_switch_field_manager, EntityTypeManagerInterface $entity_type_manager, CacheBackendInterface $cache, LoggerChannelFactoryInterface $logger) {
    $this->cache = $cache;
    $this->entityTypeManager = $entity_type_manager;
    $this->logger = $logger;
    $this->viewModeSwitchEntityFieldManager = $view_mode_switch_field_manager;
  }

  /**
   * {@inheritdoc}
   *
   * @see \Drupal\view_mode_switch\ViewModeSwitch::doGetViewModeToSwitchTo()
   */
  public function getViewModeToSwitchTo(FieldableEntityInterface $entity, $view_mode) {
    // Prepare cache ID.
    $cid = implode(':', [
      $entity->getEntityTypeId(),
      $entity->bundle(),
      $entity->id() ?: $entity->uuid(),
      $view_mode,
    ]);

    // Entity is not in preview and cached data exists?
    if (($cache = $this->cache->get($cid))) {
      return $cache->data;
    }

    // Determine view mode to switch to (if any).
    $switch_to_view_mode = $this->doGetViewModeToSwitchTo($entity, $view_mode) ?: FALSE;

    // Check for potential preview contexts.
    $is_preview_context = [
      // Is entity preview (e.g. node preview)?
      !empty($entity->in_preview),
      // Is paragraphs entity preview in experimental widget?
      $entity->getEntityTypeId() === 'paragraph' && $view_mode === 'preview',
    ];

    // Save to cache (if not in preview context).
    if (!array_filter($is_preview_context)) {
      $this->cache->set($cid, $switch_to_view_mode, Cache::PERMANENT, $entity->getCacheTags());
    }

    return $switch_to_view_mode ?: NULL;
  }

  /**
   * Internal method to get the view mode to switch to (if applicable).
   *
   * @param \Drupal\Core\Entity\FieldableEntityInterface $entity
   *   The entity to process for view mode switches.
   * @param string $view_mode
   *   The origin view mode.
   * @param array $results
   *   (internal) Used to track result information for all processed items.
   *
   * @return string|null
   *   The name of the view mode to switch to.
   *
   * @see \Drupal\view_mode_switch\ViewModeSwitch::getViewModeToSwitchTo()
   */
  protected function doGetViewModeToSwitchTo(FieldableEntityInterface $entity, $view_mode, array $results = []) {
    if (!$results) {
      $results = [
        'origin_view_mode' => $view_mode,
        'view_mode_switches' => [],
        'recursion_detected' => FALSE,
      ];
    }

    // Switch view mode (if applicable).
    if (($field = $this->viewModeSwitchEntityFieldManager->getApplicableField($entity, $view_mode))) {
      // Recursion detected?
      if (isset($results['view_mode_switches'][$field->getName()])) {
        $results['recursion_detected'] = TRUE;

        $this->logger->get('view_mode_switch')
          ->warning('Recursion detected when trying to switch %origin_view_mode view mode via %view_mode_switches.', [
            '%origin_view_mode' => $results['origin_view_mode'],
            '%view_mode_switches' => implode(' › ', [$results['origin_view_mode']] + $results['view_mode_switches'] + [$field->getViewMode()]),
          ]);

        // Return last view mode before recursion.
        return $results['view_mode_switches'][$field->getName()];
      }

      // Mark field as processed (for later recursion detection).
      $results['view_mode_switches'][$field->getName()] = $field->getViewMode();

      // Merge entity view mode cache metadata into given entity.
      $entity->addCacheableDependency($this->entityTypeManager->getStorage('entity_view_mode')->load($entity->getEntityTypeId() . '.' . $field->getViewMode()));

      // Check if there are subsequent view mode switches available for the view
      // mode currently switched to.
      if (($view_mode = $this->doGetViewModeToSwitchTo($entity, $field->getViewMode(), $results))) {
        return $view_mode;
      }

      return $field->getViewMode();
    }

    return NULL;
  }

}

==> view_mode_switch-8.x-1.2.tar.gz: view_mode_switch/src/ViewModeSwitch.php <==
<?php

namespace Drupal\view_mode_switch;

use Drupal\Core\Cache\Cache;
use Drupal\Core\Cache\CacheBackendInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Entity\FieldableEntityInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\view_mode_switch\Entity\EntityFieldManagerInterface;

/**
 * Provides the view mode switch service.
 */
class ViewModeSwitch implements ViewModeSwitchInterface {

  use StringTranslationTrait;

  /**
   * The instantiated cache backend.
   *
   * @var \Drupal\Core\Cache\CacheBackendInterface
   */
  protected $cache;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The logger channel factory.
   *
   * @var \Drupal\Core\Logger\LoggerChannelFactoryInterface
   */
  protected $logger;

  /**
   * The view mode switch entity field manager.
   *
   * @var \Drupal\view_mode_switch\Entity\EntityFieldManagerInterface
   */
  protected $viewModeSwitchEntityFieldManager;

  /**
   * Constructs a new ViewModeSwitch.
   *
   * @param \Drupal\view_mode_switch\Entity\EntityFieldManagerInterface $view_mode_switch_field_manager
   *   The view mode switch entity field manager.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\Core\Cache\CacheBackendInterface $cache
   *   A cache backend used to store view mode switches.
   * @param \Drupal\Core\Logger\LoggerChannelFactoryInterface $logger
   *   The logger channel factory.
   */
  public function __construct(EntityFieldManagerInterface $view_mode_switch_field_manager, EntityTypeManagerInterface $entity_type_manager, CacheBackendInterface $cache, LoggerChannelFactoryInterface $logger) {
    $this->cache = $cache;
    $this->entityTypeManager = $entity_type_manager;
    $this->logger = $logger;
    $this->viewModeSwitchEntityFieldManager = $view_mode_switch_field_manager;
  }

  /**
   * {@inheritdoc}
   *
   * @see \Drupal\view_mode_switch\ViewModeSwitch::doGetViewModeToSwitchTo()
   */
  public function getViewModeToSwitchTo(FieldableEntityInterface $entity, $view_mode): ?string {
    $entity_type_id = $entity->getEntityTypeId();
    $bundle = $entity->bundle();

    // Prepare cache ID.
    $cid = implode(':', [
      $entity_type_id,
      $bundle,
      $entity->id() ?: $entity->uuid(),
      $view_mode,
    ]);

    // Entity is not in preview and cached data exists?
    if (($cache = $this->cache->get($cid))) {
      return $cache->data;
    }

    // Determine view mode to switch to (if any).
    $switch_to_view_mode = $this->doGetViewModeToSwitchTo($entity, $view_mode) ?: FALSE;

    // Check for potential preview contexts.
    $is_preview_context = [
      // Is entity preview (e.g. node preview)?
      !empty($entity->in_preview),
      // Is paragraphs entity preview in experimental widget?
      $entity_type_id === 'paragraph' && $view_mode === 'preview',
    ];

    // Save to cache (if not in preview context).
    if (!array_filter($is_preview_context)) {
      $this->cache->set($cid, $switch_to_view_mode, Cache::PERMANENT, $entity->getCacheTags());
    }

    return $switch_to_view_mode ?: NULL;
  }

  /**
   * Internal method to get the view mode to switch to (if applicable).
   *
   * @param \Drupal\Core\Entity\FieldableEntityInterface $entity
   *   The entity to process for view mode switches.
   * @param string $view_mode
   *   The origin view mode.
   * @param array $results
   *   (internal) Used to track result information for all processed items.
   *
   * @return string|null
   *   The name of the view mode to switch to.
   *
   * @see \Drupal\view_mode_switch\ViewModeSwitch::getViewModeToSwitchTo()
   */
  protected function doGetViewModeToSwitchTo(FieldableEntityInterface $entity, $view_mode, array $results = []): ?string {
    if (!$results) {
      $results = [
        'origin_view_mode' => $view_mode,
        'view_mode_switches' => [],
      ];
    }

    // Switch view mode (if applicable).
    if (($field = $this->viewModeSwitchEntityFieldManager->getApplicableField($entity, $view_mode))) {
      $field_name = $field->getName();
      $target_view_mode = $field->getViewMode();

      // Recursion detected?
      if (isset($results['view_mode_switches'][$field_name])) {
        $this->logger->get('view_mode_switch')
          ->warning('Recursion detected when trying to switch %origin_view_mode view mode via %view_mode_switches.', [
            '%origin_view_mode' => $results['origin_view_mode'],
            '%view_mode_switches' => implode(' › ', [$results['origin_view_mode']] + $results['view_mode_switches'] + [$view_mode]),
            'link' => $entity->toLink($this->t('View'))->toString(),
          ]);

        // Return last view mode before recursion.
        return $results['view_mode_switches'][$field_name];
      }

      // Mark field as processed (for later recursion detection).
      $results['view_mode_switches'][$field_name] = $target_view_mode;

      // Merge entity view mode cache metadata into given entity.
      $entity->addCacheableDependency($this->entityTypeManager->getStorage('entity_view_mode')->load($entity->getEntityTypeId() . '.' . $target_view_mode));

      // Check if there are subsequent view mode switches available for the view
      // mode currently switched to.
      if (($subsequent_view_mode = $this->doGetViewModeToSwitchTo($entity, $target_view_mode, $results))) {
        return $subsequent_view_mode;
      }

      return $target_view_mode;
    }

    return NULL;
  }

}
